﻿package nicocache.action 
{
    import mx.core.mx_internal;
    import mx.controls.Alert;
    import mx.core.Application;
    import mx.rpc.AsyncDispatcher;
    import mx.rpc.AsyncToken;
    import mx.rpc.Fault;
    import mx.rpc.http.HTTPService;
    import mx.rpc.events.ResultEvent;
    import mx.rpc.events.FaultEvent;
    import flash.display.DisplayObject;
    import mx.managers.PopUpManager;
    
    import nicocache.folder.FolderInfo;
    import nicocache.Config;
    import nicocache.Constant;
    import nicocache.log.Logger;
    import nicocache.utils.NetUtil;
    import nicocache.form.FolderChooserForm;
    
    use namespace mx_internal;
    
    /**
     * 移動用HTTPServiceクラス<br/>
     * <pre>
     *   例：
     *     var moveAction:MoveAction = new MoveAction(...);
     *     moveAction.setDeaultHandler();
     *     moveAction.execute();
     * </pre>
     */
    public class MoveAction extends HTTPService
    {
        /** 動画ID */
        public var id:String;
        /** 移動元フォルダパス */
        public var srcFolder:String;
        /** 移動元フォルダパス */
        public var dstFolder:String;
        /** 対象オブジェクト */
        public var target:XML;
        /** パラメータ */
        public var parameters:Object = null;
        
        /**
         * コンストラクタ
         * 
         * @param	id 動画ID。
         * @param	srcFolder 移動元フォルダ名。
         * @param	dstFolder 移動先フォルダ名。nullの場合は、選択ダイアログが表示されます。
         * @param	target 処理対象のXMLオブジェクト
         */
        public function MoveAction(id:String, srcFolder:String, dstFolder:String=null, target:XML=null)
        {
            super();
            
            this.id = id;
            this.srcFolder = srcFolder;
            this.dstFolder = dstFolder;
            this.target = target;
            
            // 設定
            resultFormat = HTTPService.RESULT_FORMAT_TEXT;
            contentType = "application/x-www-form-urlencoded; charset=UTF-8";
            // 優先度高で登録
            addEventListener(ResultEvent.RESULT, resultEventHandler, false, int.MAX_VALUE);
        }
        
        /**
         * 移動を開始します。
         * 
         * @return トークン
         */
        public function execute():AsyncToken
        {
            var token:AsyncToken = null;
            
            // 移動先が指定されている場合
            if (dstFolder != null)
            {
                url = NetUtil.getMoveURL(id, dstFolder);
                if (check()) token = send(parameters);
            }
            // 移動先が指定されていない場合
            else
            {
                showFolderChooseDialog(function(ret:Object):void {
                    if (ret.result)
                    {
                        dstFolder = ret.path;
                        url = NetUtil.getMoveURL(id, dstFolder);
                        if (check()) token = send(parameters);
                    }
                });
            }
            
            return token;
        }
        
        /**
         * 移動元と移動先が同じかチェックします。
         * @return 同じならfalse
         */
        protected function check():Boolean
        {
            var ret:Boolean = true;
            
            if (srcFolder == dstFolder)
            {
                Logger.warn("{0} 移動元と移動先が同じです。", id);
                ret = false;
            }
            return ret;
        }
        
        /**
         * 移動先選択ダイアログを表示します。
         * @param	closeHandler ダイアログを閉じた時の動作を実装した関数
         */
        public static function showFolderChooseDialog(closeHandler:Function=null):void
        {
            var form:FolderChooserForm = PopUpManager.createPopUp(
                DisplayObject(Application.application), FolderChooserForm, true) as FolderChooserForm;
            form.init();
            form.addCloseHandler(closeHandler);
            PopUpManager.centerPopUp(form);
        }
        
        /**
         * NicoCache_nlの動画移動APIにメッセージを送信します。
         * 
         * @param	parameters パラメータ
         * @return トークン
         */
        override public function send(parameters:Object = null):AsyncToken {
            var token:AsyncToken = super.send(parameters);
            
            token.url = url;
            token.id = id;
            token.srcFolder = srcFolder;
            token.dstFolder = dstFolder;
            token.target = target;
            
            return token;
        }
        
        /**
         * デフォルトのハンドラを優先度100で登録します。
         */
        public function setDefaultHandler():void
        {
            // 優先度100で登録
            addEventListener(ResultEvent.RESULT, defaultResultHandler, false, 100);
            addEventListener(FaultEvent.FAULT, defaultFaultHandler, false, 100);
        }
        
        /**
         * 結果のメッセージが「OK」でない場合、FaultEventを送出するハンドラ。
         * @param	event イベントオブジェクト
         */
        protected function resultEventHandler(event:ResultEvent):void
        {
            var result:String = String(event.result);
            if (result.indexOf("OK") < 0)
            {
                var fault:Fault = new Fault("DELETE_ERROR", "キャッシュの移動に失敗しました。");
                var faultEvent:FaultEvent = FaultEvent.createEvent(fault, event.token);
                new AsyncDispatcher(dispatchRpcEvent, [faultEvent], 10);  // internal
                event.stopImmediatePropagation();
            }
        }
        
        /**
         * 移動成功時のデフォルトハンドラです。<br/>
         * setDefaultHandler()メソッドを呼ぶと登録されます。
         * 
         * @param	event イベントオブジェクト
         */
        protected function defaultResultHandler(event:ResultEvent):void
        {
            Logger.debug("{0} [{1}]", event.token.id, event.result as String);
            
            var app:Object = Application.application;
            var _srcFolder:String = event.token.srcFolder;
            var _dstFolder:String = event.token.dstFolder;
            if (_srcFolder == "") _srcFolder = "キャッシュファイル";
            if (_dstFolder == "") _dstFolder = "キャッシュファイル";
            
            Logger.debug("移動成功 ⇒ " + event.token.url);
            Logger.info("{0} [移動元] {1} ⇒ [移動先] {2}", event.token.id, _srcFolder, _dstFolder);
            
            var folderInfo:FolderInfo = FolderInfo.getInstance();
            // 現在表示しているフォルダが移動元の場合
            if (folderInfo.path == event.token.srcFolder)
            {
                if (app.hasOwnProperty("removeListData"))
                    app.removeListData(event.token.id);
            }
            // 現在表示しているフォルダが移動先の場合
            if (folderInfo.path == event.token.dstFolder)
            {
                if (app.hasOwnProperty("addListData") && event.token.target)
                    app.addListData(XML(event.token.target));
            }
        }
        
        /**
         * 移動失敗時のデフォルトハンドラです。<br/>
         * setDefaultHandler()メソッドを呼ぶと登録されます。
         * 
         * @param	event イベントオブジェクト
         */
        protected function defaultFaultHandler(event:FaultEvent):void
        {
            Logger.error("移動失敗 ⇒ " + event.token.url);
        }
    }
    
}