﻿package nicocache.controls
{
    import nicocache.Config;
    import nicocache.Constant;
    import nicocache.controls.columnlist.*;
    import nicocache.folder.FolderInfo;

    /**
     * ファイル一覧を表示するDataGridのカラムを管理するクラス
     */
    public final class ColumnManager
    {
        /** Singletonクラスのインスタンス */
        private static var instance:ColumnManager = null;
        /** Configクラス */
        private var config:Config = Config.getInstance();
        /** キー⇒ColumnList 変換テーブル */
        private var columnsMap:Object = new Object();

        /**
         * コンストラクタ
         *
         * @param blocker ブロック
         */
        public function ColumnManager(blocker:Blocker)
        {
            if (blocker == null) throw new Error("getInstance()を使用してください。");
        }

        /**
         * インスタンスを取得します。
         *
         * @return インスタンス
         */
        public static function getInstance():ColumnManager 
        {
            if (instance == null) instance = new ColumnManager(new Blocker());
            return instance;
        }
        
        /**
         * キーワードを指定して、カラムリストを登録します。
         *
         * @param key キーワード
         * @param columns カラム
         */
        public function addColumns(key:String, columns:DefaultColumnList):void
        {
            columnsMap[key] = columns;
        }

        /**
         * カラムリストを取得します。
         *
         * @param key キーワード
         * @return カラムリスト
         */
        public function getColumns(folderInfo:FolderInfo):Array
        {
            var key:String = folderInfo.view + ":" + folderInfo.path;
            var obj:DefaultColumnList = columnsMap[key] as DefaultColumnList;
            if (obj == null)
            {
                obj = createColumnList(folderInfo.view, folderInfo.path);
                addColumns(key, obj);
            }
            return obj.columns
        }
        
        /**
         * カラムリストを生成します。
         * 
         * @param	view ビュー
         * @param	path パス
         * @return カラムリスト
         */
        protected function createColumnList(view:String, path:String):DefaultColumnList
        {
            if (view == Constant.VIEW_CACHE)  return new CacheColumnList(view, path);
            if (view == Constant.VIEW_TEMP) return new TempColumnList(view, path);
            if (view == Constant.VIEW_ALL) return new AllColumnList(view, path);
            throw new Error("引数が正しくありません。");
        }
    }
}

/**
 * ブロッククラス
 *    http://blogs.adobe.com/kiwi/2006/05/as3_programming_101_for_cc_cod_1.html#more
 *
 */
class Blocker 
{
}
