﻿package nicocache.controls
{
    import flash.ui.ContextMenu;
    import flash.ui.ContextMenuItem;
    import flash.events.MouseEvent;
    import flash.events.Event;
    import flash.events.ContextMenuEvent;
    import flash.system.System;
    import nicocache.Config;
    
    import mx.core.Application;
    import mx.controls.Label;
    import mx.controls.listClasses.IListItemRenderer;
    import mx.containers.HBox;
    import mx.utils.StringUtil

    import nicocache.log.Logger;
    import nicocache.controls.ImageButton;
    import nicocache.utils.NetUtil;
    import nicocache.utils.BooleanUtil;
    import nicocache.form.FolderChooserForm;
    import nicocache.folder.FolderInfo;
    import nicocache.Constant;
    import nicocache.action.DeleteAction;
    import nicocache.action.MoveAction;
    import nicocache.utils.StringTool;

    public class ImageButtonItemRenderer extends HBox implements IListItemRenderer
    {
        // 画像埋め込み
        [Embed(source="../../img/save.gif")]
        private static var DownloadImage:Class;
        [Embed(source="../../img/audio.gif")]
        private static var ExtractAudioImage:Class;
        [Embed(source="../../img/move.gif")]
        private static var MoveImage:Class;
        [Embed(source="../../img/del.gif")]
        private static var DeleteImage:Class;

        private var btnDownload:ImageButton;
        private var btnExtractAudio:ImageButton;
        private var btnDelete:ImageButton;
        private var btnMove:ImageButton;
        private var lblId:Label;

        private var columnData:Object;

        /** キャッシュID */
        private var cacheId:String;
        /** 動画ID */
        private var movieId:String;
        /** フォーマット */
        private var format:String;
        /** エコノミー */
        private var economy:String;
        /** 一時ファイルフラグ */
        private var tempFile:Boolean;

        /**
         * コンストラクタ
         * 
         */
        public function ImageButtonItemRenderer()
        {
            super();

            setStyle("paddingLeft", "2");
            setStyle("horizontalGap", "1");
            // セルにスクロールバーを表示しない
            horizontalScrollPolicy = "off";
            verticalScrollPolicy = "off";

            btnDownload = new ImageButton();
            btnDownload.toolTip = "保存";
            btnDownload.source = new DownloadImage();
            btnDownload.contextMenu = createDownloadContextMenu();
            btnDownload.addEventListener(MouseEvent.CLICK, openDownloadLink);

            btnExtractAudio = new ImageButton();
            btnExtractAudio.toolTip = "音声抽出";
            btnExtractAudio.source = new ExtractAudioImage();
            btnExtractAudio.contextMenu = createExtractAudioContextMenu();
            btnExtractAudio.addEventListener(MouseEvent.CLICK, openExtractAudioLink);

            btnDelete = new ImageButton();
            btnDelete.toolTip = "削除";
            btnDelete.source = new DeleteImage();
            btnDelete.addEventListener(MouseEvent.CLICK, clickDeleteButton);

            btnMove = new ImageButton();
            btnMove.toolTip = "移動";
            btnMove.source = new MoveImage();
            btnMove.addEventListener(MouseEvent.CLICK, clickMoveButton);

            lblId = new Label();
            lblId.addEventListener(MouseEvent.CLICK, clickPalyLinkLabel);
            lblId.styleName = "gridText";
            
            // クリック時の動作
            addEventListener(MouseEvent.CLICK, function(event:MouseEvent):void {
                var _data:Object = data;
                if (_data != null)
                {
                    var id:String = String(_data[Constant.DATAFIELD_ID]);
                    id = StringTool.altid2smid(id);
                    Application.application.setShowThumbInfo(id);
                }
            });
        }

        /**
         * 保存リンクを右クリックした時に表示するコンテキストを作成します。
         * 
         * @return コンテキストメニュー
         */
        private function createDownloadContextMenu():ContextMenu
        {
            return createContextMenu(openDownloadLink, openDownloadLinkAsNew, copyDownloadLink);
        }

        /**
         * 音声抽出リンクを右クリックした時に表示するコンテキストを作成します。
         * 
         * @return コンテキストメニュー
         */
        private function createExtractAudioContextMenu():ContextMenu
        {
            return createContextMenu(openExtractAudioLink, openExtractAudioLinkAsNew, copyExtractAudioLink);
        }

        /**
         * コンテキストメニューを作成します。
         * 
         * @param	openFunc 「開く」メニューの処理を実装した関数
         * @param	openAsNewFunc 「新しいウィンドウで開く」メニューの処理を実装した関数
         * @param	linkCopyFunc 「リンクのコピー」メニューの処理を実装した関数
         * @return コンテキストメニュー
         */
        private function createContextMenu(openFunc:Function, openAsNewFunc:Function, linkCopyFunc:Function):ContextMenu
        {
            var menu:ContextMenu;
            var item:ContextMenuItem;

            menu = new ContextMenu();
            menu.hideBuiltInItems();

            item = new ContextMenuItem("開く");
            item.addEventListener(ContextMenuEvent.MENU_ITEM_SELECT, openFunc);
            menu.customItems.push(item);
            item = new ContextMenuItem("新規ウィンドウで開く");
            item.addEventListener(ContextMenuEvent.MENU_ITEM_SELECT, openAsNewFunc);
            menu.customItems.push(item);
            item = new ContextMenuItem("リンクのコピー");
            item.addEventListener(ContextMenuEvent.MENU_ITEM_SELECT, linkCopyFunc);
            menu.customItems.push(item);

            return menu;
        }

        /**
         * dataプロパティをオーバーライド
         * 
         */
        override public function get data():Object
        {
            return columnData;
        }
        
        /**
         * dataプロパティをオーバーライド
         * 
         */
        override public function set data(value:Object):void
        {
            columnData = value;
            if (columnData != null)
            {
                cacheId  = columnData[Constant.DATAFIELD_ID];
                movieId  = StringTool.altid2smid(cacheId);
                format   = columnData[Constant.DATAFIELD_FORMAT];
                economy  = columnData[Constant.DATAFIELD_ECONOMY];
                economy  = BooleanUtil.parseBoolean(economy) ? "low" : "";
                tempFile = BooleanUtil.parseBoolean(columnData[Constant.DATAFIELD_TEMPFILE]);
                
                lblId.htmlText = StringUtil.substitute(
                    "<a href=\"http://www.nicovideo.jp/watch/{0}\">{1}</a>", movieId, cacheId);
                
                // 全てのアイテムを一度削除
                removeAllChildren();
                // キャッシュファイルの場合
                if (!tempFile)
                {
                    addChild(btnDownload);
                    addChild(btnExtractAudio);
                    addChild(btnDelete);
                    addChild(btnMove);
                }
                // 一時ファイルの場合
                else
                {
                    addChild(btnDelete);
                }
                // IDラベル追加
                addChild(lblId);
            }
        }

        /**
         * 動画再生ページを現在のページで開きます。
         *
         * @param event イベントオブジェクト
         */
        private function clickPalyLinkLabel(event:MouseEvent):void
        {
            var target:String = Config.getInstance().alwaysOpenNewWindow ? "_blank" : "_self";
            if (movieId) NetUtil.windowOpen("http://www.nicovideo.jp/watch/" + movieId, target);
        }

        /**
         * 音声抽出リンクを現在のページで開きます。
         * 
         * @param	event イベントオブジェクト
         */
        private function openExtractAudioLink(event:Event):void
        {
            execExtractAudio();
        }

        /**
         * 音声抽出リンクを新しいウィンドウで開きます。
         * 
         * @param	event イベントオブジェクト
         */
        private function openExtractAudioLinkAsNew(event:Event):void
        {
            execExtractAudio("_blank");
        }

        /**
         * 音声抽出リンクを開きます。
         * 
         * @param target 対象ウィンドウ[デフォルトで現在のウィンドウ]
         */
        private function execExtractAudio(target:String="_self"):void
        {
            var url:String = getExtractAudioURL();
            if (!url) return;
            NetUtil.windowOpen(url, target);
            Logger.info("音声ファイル抽出 ⇒ " + url);
        }

        /**
         * ファイルダウンロード用のURLを取得します。
         *
         * @return URL
         */
        private function getDownloadURL():String
        {
            return NetUtil.getDownloadURL(cacheId, format);
        }

        /**
         * 音声ファイル抽出用のURLを取得します。
         *
         * @return URL
         */
        private function getExtractAudioURL():String
        {
            return NetUtil.getExtractAudioURL(cacheId, format);
        }

        /**
         * ダウンロードページを現在のウィンドウで開きます。
         *
         * @param event イベントオブジェクト
         */
        private function openDownloadLink(event:Event):void
        {
            execDownload();
        }

        /**
         * ダウンロードページを新しいウィンドウで開きます。
         *
         * @param event イベントオブジェクト
         */
        private function openDownloadLinkAsNew(event:Event):void
        {
            execDownload("_target");
        }

        /**
         * ダウンロードページを開きます。
         * 
         * @param	target 対象ウィンドウ[デフォルトで現在のウィンドウ]
         */
        private function execDownload(target:String="_self"):void
        {
            if (!movieId) return;

            var url:String = getDownloadURL();
            NetUtil.windowOpen(url, target);
            Logger.info("動画ファイル保存 ⇒ " + url);
        }

        /**
         * 音声抽出ページのリンクをクリップボードにコピーします。
         * 
         * @param	event イベントオブジェクト
         */
        private function copyExtractAudioLink(event:Event):void
        {
            var url:String = getExtractAudioURL();
            if (!url) return;

            System.setClipboard(url);
        }

        /**
         * ダウンロードページのリンクをクリップボードにコピーします。
         * 
         * @param	event イベントオブジェクト
         */
        private function copyDownloadLink(event:Event):void
        {
            if (!movieId) return;

            System.setClipboard(getDownloadURL());
        }

        /**
         * 削除ボタンを押された時の処理を実装します。
         * 
         * @param	event イベントオブジェクト
         */
        private function clickDeleteButton(event:Event):void
        {
            var folderPath:String = FolderInfo.getInstance().path;
            
            var deleteAction:DeleteAction = new DeleteAction(cacheId, folderPath, tempFile, (economy == "low"));
            deleteAction.setDefaultHandler();
            deleteAction.execute();
        }

        /**
         * 移動ボタンが押された時の処理を実装します。
         * 
         * @param	event イベントオブジェクト
         */
        private function clickMoveButton(event:Event):void
        {
            var folderInfo:FolderInfo = FolderInfo.getInstance();
            var moveAction:MoveAction = new MoveAction(cacheId, folderInfo.path);
            moveAction.setDefaultHandler();
            moveAction.execute();
        }
    }
}
