﻿package nicocache.controls.column
{
    import nicocache.Constant;
    import nicocache.controls.column.CustomColumn;
    import nicocache.controls.NumberItemRenderer;
    import mx.controls.dataGridClasses.DataGridColumn;
    import mx.core.ClassFactory;
    import mx.formatters.NumberFormatter;
    import mx.formatters.NumberBaseRoundType;

    /**
     * サイズカラム
     *
     */
    public class SizeColumn extends CustomColumn
    {
        protected static var formatter:NumberFormatter = new NumberFormatter();
        
        /**
         * コンストラクタ
         *
         */
        public function SizeColumn()
        {
            super();
            // デフォルト値設定
            dataField           = Constant.DATAFIELD_SIZE;
            headerText          = "サイズ";
            width               = 70;
            minWidth = 50;
            resizable           = true;
            sortable            = true;
            itemRenderer        = new ClassFactory(NumberItemRenderer);
            sortCompareFunction = sizeSortFunction;
            labelFunction = sizeLabelFunction;
            dataTipField = Constant.DATAFIELD_SIZE;
            showDataTips     = false;
        }

        /**
         * ファイルサイズの大小を比較します。
         *
         * @param obj1 比較対象オブジェクト1
         * @param obj2 比較対象オブジェクト2
         * @return 同じなら0、小さい場合は-1、大きい場合は1を返す
         */
        protected function sizeSortFunction(obj1:Object, obj2:Object):int
        {
            var id1:Number = Number(XMLList(obj1).@size);
            var id2:Number = Number(XMLList(obj2).@size);
            return compareValues(id1, id2);
        }
        
        /**
         * ラベルの生成を行います。
         *
         * @param item 対象セルの行データ
         * @param column 対象セルの列データ
         * @return ラベル
         */
        protected function sizeLabelFunction(item:Object, column:DataGridColumn):String
        {
            var label:String = "";

            if (item != null)
            {
                formatter.rounding = NumberBaseRoundType.NEAREST;
                label = formatter.format(int(item[column.dataField]) / 1024) + " KB";
            }
            return label;
        }
    }
}

